import { useInfoStore } from '@/stores/Info'
import axios from 'axios'
import type {
  AxiosResponse,
  Method,
  AxiosError,
  AxiosRequestConfig
} from 'axios'

import { showMessage } from './status'
import type { IResponse } from './type'

axios.defaults.timeout = 5000
axios.defaults.headers.post['Access-Control-Allow-Origin-Type'] = '*'

const baseApi = import.meta.env.VITE_APP_BASE_URL
const URL = import.meta.env.VITE_APP_API_SERVER
// 方便后续上传图片
export const baseURL = (URL as string) + baseApi

const axiosInstance = axios.create({
  baseURL: '/api'
})

axiosInstance.interceptors.response.use(
  (response: AxiosResponse) => {
    const res = response.data
    if (res.code === 200) {
      return res
    } else {
      showMessage(res.code)
      return Promise.reject(res)
    }
  },
  (error: AxiosError) => {
    const { response } = error
    if (response) {
      // 请求已发出，但是不在2xx的范围
      showMessage(response.status)
      return Promise.reject(response.data)
    }
    showMessage('网络连接异常,请稍后再试!')
  }
)

// 后续白名单
const whiteList = ['']
axiosInstance.interceptors.request.use(
  (config: AxiosRequestConfig) => {
    const user = useInfoStore()
    const { token } = user.userinfo
    if (token && config.headers && !whiteList.includes(config.url as string)) {
      config.headers.token = token
    }
    return config
  },
  (error: AxiosError) => {
    return Promise.reject(error)
  }
)

/**
 * @param {String} - url  请求地址
 * @param {String} - method  请求类型
 * @param {Object} - submitData  对象类型，提交数据
 */
export const server = <T = any>(
  url: string,
  method: Method,
  submitData?: unknown
) => {
  return axiosInstance.request<T, IResponse<T>>({
    url,
    method,
    [method.toLowerCase() === 'get' ? 'params' : 'data']: submitData
  })
}

export default axiosInstance
